package hal

import (
	"strings"
)

type ProgrammedInstruction struct {
	Instruction *Instruction
	Operand     float64
}

func (pi *ProgrammedInstruction) Execute(module *Module) {
	if pi.Instruction.ExecuteWithOperand != nil {
		module.debug("Operand: %d", pi.Operand)
		pi.Instruction.ExecuteWithOperand(module, pi.Operand)
	} else if pi.Instruction.Execute != nil {
		pi.Instruction.Execute(module)
	} else {
		panic("instruction is not implemented")
	}
}

type Instruction struct {
	Name               string
	ExecuteWithOperand func(module *Module, operand float64)
	Execute            func(module *Module)
}

func FindInstructionByName(name string) *Instruction {
	for _, instruction := range instructions {
		if instruction.Name == strings.ToUpper(name) {
			return instruction
		}
	}
	return nil
}

var instructions = []*Instruction{
	InstructionStart,
	InstructionStop,
	InstructionIn,
	InstructionOut,
	InstructionAddnum,
}

var InstructionStart = &Instruction{
	Name: "START",
	Execute: func(module *Module) {
		// actually not required
		module.isStopped = false
	},
}

var InstructionStop = &Instruction{
	Name: "STOP",
	Execute: func(module *Module) {
		module.isStopped = true
	},
}

var InstructionOut = &Instruction{
	Name: "OUT",
	ExecuteWithOperand: func(module *Module, operand float64) {
		module.IO[int64(operand)] = module.accumulator
	},
}

var InstructionIn = &Instruction{
	Name: "IN",
	Execute: func(module *Module) {
		// TODO implement
	},
}

var InstructionAddnum = &Instruction{
	Name: "ADDNUM",
	ExecuteWithOperand: func(module *Module, operand float64) {
		module.accumulator = module.accumulator + operand
	},
}
